#pragma once

// GLEW
#include <GL/glew.h>

// SDL
#include <SDL.h>
#include <SDL_opengl.h>

// Utils
#include "gVertexBuffer.h"
#include "gShaderProgram.h"

// CL
#include <iostream>
#include <fstream>
#include <sstream>

#define __NO_STD_VECTOR
#define __CL_ENABLE_EXCEPTIONS

#ifdef __APPLE__
#include <CL/cl.hpp>
#else
#include <CL/cl.hpp>
#include <CL/cl_gl.h>
#endif

#ifdef __GNUC__
#include <GL/glx.h>
#endif

class CMyApp
{
public:
	CMyApp(void);
	~CMyApp(void);

	bool InitGL();
	bool InitCL();

	void Clean();

	void Update();
	void Render();

	void KeyboardDown(SDL_KeyboardEvent&);
	void KeyboardUp(SDL_KeyboardEvent&);
	void MouseMove(SDL_MouseMotionEvent&);
	void MouseDown(SDL_MouseButtonEvent&);
	void MouseUp(SDL_MouseButtonEvent&);
	void MouseWheel(SDL_MouseWheelEvent&);
	void Resize(int, int);
protected:

	// GL
	int windowH, windowW;
	GLuint m_vaoID, vbo, texture;
	void renderVBO( int vbolen );

	// CL
	cl::Context context;
	cl::CommandQueue command_queue;
	cl::Program program;

	cl::Kernel kernel_update;

	cl::BufferGL cl_vbo_mem;
	cl::Buffer cl_v, cl_m;

	float delta_time;

	const int num_particles = 25000;
	const float particle_size = 0.01f;
	const bool bRing = true;
	const bool bRandVelocities = true;
	const float massiveObjectMass = 1;

#pragma region GL functions

	gShaderProgram	m_program;
	GLuint m_textureID;

	GLuint initVBO(int vbolen )
	{
		GLuint vbo_buffer;
		// generate the buffer
		glGenBuffers(1, &vbo_buffer);

		// bind the buffer
		glBindBuffer(GL_ARRAY_BUFFER, vbo_buffer);
		glBufferData(GL_ARRAY_BUFFER, vbolen*sizeof(float)*2, 0, GL_DYNAMIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0); 

		return vbo_buffer;
	}

#pragma endregion

};